/*
Copyright (c) 2010, Геннадий Берман (http://genberm.narod.ru)
All rights reserved.

Данное программное обеспечение предоставляется на условиях "AS IS".
Разрешается свободное использование, распространение и модификация программы,
с целью ее дальнейшего использования, при условии соблюдения вышеупомянутого
авторского права.

Никакие гарантии не обеспечиваются. Никакой ответственности за прямые,
косвенные или любые другие убытки, возникшие из-за использования данного
программного обеспечения, автор не несет.
*/
#include "mainwindow.h"
#include <QVBoxLayout>
#include <QHBoxLayout>


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    if (objectName().isEmpty())
        setObjectName(tr("MainWindow"));
    //формируем представление главного окна
    QWidget * centralWidget = new QWidget(this);
    centralWidget->setObjectName(tr("centralWidget"));
    setCentralWidget(centralWidget);

    QGridLayout * gridLayout = new QGridLayout;
    centralWidget->setLayout(gridLayout);

    //создаем модель заголовка
    QStandardItemModel *headerModel = new QStandardItemModel;
    //вызываем функцию формирования заголовка
    BuildHeaderModel(*headerModel);
    //создаем таблицу
    complexTableView = new QgebComplexTableView(this);
    complexTableView->setObjectName("ComplexTableView");  //объектное имя таблицы
    complexTableView->setVisible(true);
    complexTableView->setSortingEnabled(true);

    //передаем в класс модель заголовка (должна быть передана до установки непрокручиваемых колонок)
    complexTableView->setHeaderModel(headerModel);
    //устанавливаем модель данных
    QStandardItemModel * stm = new QStandardItemModel;
    complexTableView->setModel(stm);

    //строим для примера таблицу в 9 колонок и 15 рядов
    complexTableView->model()->insertColumns(0,9, QModelIndex());
    complexTableView->model()->insertRows(0,15, QModelIndex());
    //задаем вектора для размещения признаков выравнивания для представления таблицы на печати и в HTML файле
    //вектор для хранения значений горизонтального выравнивания в колонках
    QVector <int> alignColumnText;
    //вектор для хранения значений вертикального выравнивания в колонках
    QVector <int> valignColumnText;
    //заполняем таблицу
    for (int row=0; row<complexTableView->model()->rowCount(QModelIndex()); ++row)
    {
        for (int col=0; col<complexTableView->model()->columnCount(QModelIndex()); ++col)
        {
            if ( row == 0 )
            {
                //здесь для примера устанавливаем явно, если это не сделать, то умолчанию (left и top)
                //устанавливаем для всех колонок значение горизонтального и вертикального выравнивания
                //при представлении таблицы на печати и в HTML файле
                //(делаем один раз)
                alignColumnText.append(Qt::AlignHCenter);
                valignColumnText.append(Qt::AlignVCenter);
            }
            QModelIndex index = complexTableView->model()->index(row, col, QModelIndex());
            //в строке 2 колонка 1 вводим более сложную информацию из 4-х строк, чтобы показать возможности выравнивания
            if ( (row==2) && (col==1) )
            {
                QString ss = QString::number(row)+"_"+QString::number(col);
                ss = ss + "\n" + ss + "\n" + ss + "\n" + ss;
                complexTableView->model()->setData(index, QVariant(ss));
            }
            else
                //стандартная информация, помещаемая в ячейку тела таблицы
                complexTableView->model()->setData(index, QVariant(QString::number(row)+"_"+QString::number(col)));
        }
    }

    //устанавливаем значения выравнивания в колонках при представлении таблицы на печати и в HTML файле
    complexTableView->setAlignColumn(alignColumnText);
    complexTableView->setVAlignColumn(valignColumnText);

    //делаем левые две колонки не прокручиваемыми
    complexTableView->setFrozenColumn(2);

    //пример синхронизации представления прокручиваемой и не прокручиваемой таблицы
    //устанавливаем для вертикального заголовка изменение высоты в зависимости от данных
    complexTableView->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);
    //получаем не прокручиваемую ячейку и устанавливаем для ее вертикального заголовка изменение высоты в зависимости от данных
    complexTableView->getFrozenTable()->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);

    //разрешаем в случае необходимости горизонтальную и вертикальную прокрутку
    complexTableView->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    complexTableView->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);

    //восстанавливаем сохраненные значения
    complexTableView->restoreStateTable("c:/complexTableView.xml");

    //размещаем таблицу
    gridLayout->addWidget(complexTableView, 0, 0);
}

MainWindow::~MainWindow()
{
    int ret = complexTableView->saveStateTable("c:/complexTableView.xml");
    //проверяем, успешно ли сохранились параметры таблицы
    if ( ret != 0 )
        QMessageBox::critical(this, tr("Error"), tr("Ошибка сохранения параметров таблицы. Код: ") + QString::number(ret), QMessageBox::Ok);
    //уничтожаем таблицу
    delete complexTableView;
}

/*
  Построение заголовка таблицы
*/
void MainWindow::BuildHeaderModel(QStandardItemModel& headerModel)
{
    //Заголовки не прокручиваемых колонок. В них не должно быть иерархии, т.е. они всегда на уровне 1.
    QStandardItem* fixedItem = new QStandardItem("Фиксированная\nколонка 1");
    QStandardItem* fixedItem1 = new QStandardItem("Фиксированная\nколонка 2");

    //Задаем еще два заголовка уровня 1. Для первого будем формировать иерархию, для второго нет.
    QStandardItem* rootItem = new QStandardItem("Уровень 1 с иерархией");
    QStandardItem* rootItem1 = new QStandardItem("Уровень 1\nбез иерархии");

    //список для элементов заголовков, находящихся под rootItem (уровень 2)
    QList<QStandardItem*> list;

    //Создаем элемент уровеня 2 под rootItem. Дочерних не имеет. Название устанавливаем вертикально (с поворотом)
    QStandardItem* rotatedTextCell=new QStandardItem("Уровень 2\nВращаемый текст");
    rotatedTextCell->setData(1, Qt::UserRole);  //Признак поворота: первый параметр равен 1
    list.push_back(rotatedTextCell);            //Вставляем значение в конец списка
    rootItem->appendColumn(list);               //Добавляем колонку, содержащую элементы

    //Создаем элемент уровня 2 под rootItem. Дочерних не имеет.
    list.clear();
    list.push_back(new QStandardItem("Уровень 2\nОбычный текст"));
    rootItem->appendColumn(list);

    //Создаем элемент уровня 2 под rootItem. Имеет дочерние.
    list.clear();
    QStandardItem* cell=new QStandardItem("Уровень 2\nС дочерними");
    list.push_back(cell);
    rootItem->appendColumn(list);

    //Создаем элемент уровеня 3 под cell.  Дочерних не имеет.
    list.clear();
    list.push_back(new QStandardItem("Уровень 3\nБез дочерних"));
    cell->appendColumn(list);

    //Создаем элемент уровеня 3 под cell.  Имеет дочерние.
    list.clear();
    QStandardItem* cell3=new QStandardItem("Уровень 3\nС дочерними");
    list.push_back(cell3);
    cell->appendColumn(list);

    //Создаем элемент уровеня 4 под cell3.  Дочерних не имеет.
    list.clear();
    list.push_back(new QStandardItem("Уровень 4\nПервый элемент"));
    cell3->appendColumn(list);

    //Создаем элемент уровеня 4 под cell3.  Дочерних не имеет.
    list.clear();
    list.push_back(new QStandardItem("Уровень 4\nВторой элемент"));
    cell3->appendColumn(list);

    //Создаем элемент уровеня 2 под rootItem. Дочерних не имеет.
    list.clear();
    list.push_back(new QStandardItem("Уровень 2\nБез дочерних"));
    rootItem->appendColumn(list);

    //Устанавливаем элементы уровня 1 в модель заголовка
    headerModel.setItem(0, 0, fixedItem);
    headerModel.setItem(0, 1, fixedItem1);
    headerModel.setItem(0, 2, rootItem);
    headerModel.setItem(0, 3, rootItem1);
}
