/*
Copyright (c) 2010, Геннадий Берман (http://genberm.narod.ru)
All rights reserved.

-------------------------------------------------------------------------
Сcылки.
При разработке данного класса заимствованы:
- фрагмент кода из программы построения таблицы со сложным заголовком,
написанной Петром Краснощековым (Krasnoshchekov Petr);
- идея создания не прокручиваемой колонки из программы frozencolumn,
являющейся одним из примеров, поставляемых со свободно распространяемой
библиотекой Qt фирмы Nokia.
-------------------------------------------------------------------------

Данное программное обеспечение предоставляется на условиях "AS IS".
Разрешается свободное использование, распространение и модификация программы,
с целью ее дальнейшего использования, при условии соблюдения вышеупомянутого
авторского права.

Никакие гарантии не обеспечиваются. Никакой ответственности за прямые,
косвенные или любые другие убытки, возникшие из-за использования данного
программного обеспечения, автор не несет.
*/
#include <QPainter>
#include <QAbstractItemModel>
#include <QPointer>

#include "qgebcomplextableview.h"
#include <QtGui>
#include <QFile>
#include <QWebView>

ProxyModelWithHeaderModels::ProxyModelWithHeaderModels(QObject* parent)
        : QProxyModel(parent)
{}

QVariant ProxyModelWithHeaderModels::data(const QModelIndex& index, int role) const
{
        if(_horizontalHeaderModel && role==HierarchicalHeaderView::HorizontalHeaderDataRole)
        {
                QVariant v;
                v.setValue((QObject*)_horizontalHeaderModel.data());
                return v;
        }
        return QProxyModel::data(index, role);
}

void ProxyModelWithHeaderModels::setHorizontalHeaderModel(QAbstractItemModel* headerModel)
{
        _horizontalHeaderModel = headerModel;
        int cnt = model()->columnCount();
        if(cnt)
                emit headerDataChanged(Qt::Horizontal, 0, cnt-1);
}


class HierarchicalHeaderView::private_data
{
        public:
                QPointer<QAbstractItemModel> headerModel;

                private_data()
                {}

                void initFromNewModel(QAbstractItemModel* model)
                {
                        headerModel=QPointer<QAbstractItemModel>();
                        QVariant v(model->data(
                                QModelIndex(),
                                HorizontalHeaderDataRole));
                        if(v.isValid())
                                headerModel = qobject_cast<QAbstractItemModel*>(v.value<QObject*>());
                }

                QModelIndex findRootIndex(QModelIndex index) const
                {
                        while(index.parent().isValid())
                                index=index.parent();
                        return index;
                }

                QModelIndexList parentIndexes(QModelIndex index) const
                {
                        QModelIndexList indexes;
                        while(index.isValid())
                        {
                                indexes.push_front(index);
                                index=index.parent();
                        }
                        return indexes;
                }


                QModelIndex findLeaf(const QModelIndex& curentIndex, int sectionIndex, int& curentLeafIndex)
                {
                        if(curentIndex.isValid())
                        {
                                int childCount=curentIndex.model()->columnCount(curentIndex);
                                if(childCount)
                                {
                                        for(int i=0; i<childCount; ++i)
                                        {
                                                QModelIndex res(findLeaf(curentIndex.child(0, i), sectionIndex, curentLeafIndex));
                                                if(res.isValid())
                                                        return res;
                                        }
                                }
                                else
                                {
                                        ++curentLeafIndex;
                                        if(curentLeafIndex==sectionIndex)
                                                return curentIndex;
                                }
                        }
                        return QModelIndex();
                }


                QModelIndex leafIndex(int sectionIndex)
                {
                        if(headerModel)
                        {
                                int curentLeafIndex=-1;
                                for(int i=0; i<headerModel->columnCount(); ++i)
                                {
                                        QModelIndex res(findLeaf(headerModel->index(0, i), sectionIndex, curentLeafIndex));
                                        if(res.isValid())
                                                return res;
                                }
                        }
                        return QModelIndex();
                }


                QModelIndexList searchLeafs(const QModelIndex& curentIndex) const
                {
                        QModelIndexList res;
                        if(curentIndex.isValid())
                        {
                                int childCount=curentIndex.model()->columnCount(curentIndex);
                                if(childCount)
                                {
                                        for(int i=0; i<childCount; ++i)
                                                res+=searchLeafs(curentIndex.child(0, i));
                                }
                                else
                                        res.push_back(curentIndex);
                        }
                        return res;
                }


                QModelIndexList leafs(const QModelIndex& searchedIndex) const
                {
                        QModelIndexList leafs;
                        if(searchedIndex.isValid())
                        {
                                int childCount=searchedIndex.model()->columnCount(searchedIndex);
                                for(int i=0; i<childCount; ++i)
                                        leafs+=searchLeafs(searchedIndex.child(0, i));
                        }
                        return leafs;
                }



                void setForegroundBrush(QStyleOptionHeader& opt, const QModelIndex& index) const
                {
                        QVariant foregroundBrush = index.data(Qt::ForegroundRole);
                        if (qVariantCanConvert<QBrush>(foregroundBrush))
                                opt.palette.setBrush(QPalette::ButtonText, qvariant_cast<QBrush>(foregroundBrush));
                }

                void setBackgroundBrush(QStyleOptionHeader& opt, const QModelIndex& index) const
                {
                        QVariant backgroundBrush = index.data(Qt::BackgroundRole);
                        if (qVariantCanConvert<QBrush>(backgroundBrush))
                        {
                                opt.palette.setBrush(QPalette::Button, qvariant_cast<QBrush>(backgroundBrush));
                                opt.palette.setBrush(QPalette::Window, qvariant_cast<QBrush>(backgroundBrush));
                        }
                }



                QSize cellSize
                        (const QModelIndex& leafIndex,
                         const QHeaderView* hv,
                         QStyleOptionHeader styleOptions) const
                {
                        QSize res;
                        QVariant variant(leafIndex.data(Qt::SizeHintRole));
                        if (variant.isValid())
                                res=qvariant_cast<QSize>(variant);
                        QFont fnt(hv->font());
                        QVariant var(leafIndex.data(Qt::FontRole));
                        if (var.isValid() && qVariantCanConvert<QFont>(var))
                                fnt = qvariant_cast<QFont>(var);
                        fnt.setBold(true);
                        QFontMetrics fm(fnt);
                        QSize size(fm.size(0, leafIndex.data(Qt::DisplayRole).toString()));
                        if(leafIndex.data(Qt::UserRole).isValid())
                                size.transpose();
                        QSize decorationsSize(hv->style()->sizeFromContents(QStyle::CT_HeaderSection, &styleOptions, QSize(), hv));
                        QSize emptyTextSize(fm.size(0, ""));
                        return res.expandedTo(size+decorationsSize-emptyTextSize);
                }

                int	currentCellWidth
                        (const QModelIndex& searchedIndex,
                         const QModelIndex& leafIndex,
                         int sectionIndex,
                         const QHeaderView* hv) const
                {
                        QModelIndexList leafsList(leafs(searchedIndex));
                        if(leafsList.empty())
                                return hv->sectionSize(sectionIndex);
                        int width=0;
                        int firstLeafSectionIndex=sectionIndex-leafsList.indexOf(leafIndex);
                        for(int i=0; i<leafsList.size(); ++i)
                                width+=hv->sectionSize(firstLeafSectionIndex+i);
                        return width;
                }

                int	currentCellLeft
                        (const QModelIndex& searchedIndex,
                         const QModelIndex& leafIndex,
                         int sectionIndex,
                         int left,
                         const QHeaderView* hv) const
                {
                        QModelIndexList leafsList(leafs(searchedIndex));
                        if(!leafsList.empty())
                        {
                                int n=leafsList.indexOf(leafIndex);
                                int firstLeafSectionIndex=sectionIndex-n;
                                --n;
                                for(; n>=0; --n)
                                        left-=hv->sectionSize(firstLeafSectionIndex+n);
                        }
                        return left;
                }



                int paintHorizontalCell
                        (QPainter *painter,
                         const QHeaderView* hv,
                         const QModelIndex& cellIndex,
                         const QModelIndex& leafIndex,
                         int logicalLeafIndex,
                         const QStyleOptionHeader& styleOptions,
                         const QRect& sectionRect,
                         int top) const
                {
                        QStyleOptionHeader uniopt(styleOptions);
                        setForegroundBrush(uniopt, cellIndex);
                        setBackgroundBrush(uniopt, cellIndex);

                        int height=cellSize(cellIndex, hv, uniopt).height();
                        if(cellIndex==leafIndex)
                                height=sectionRect.height()-top;
                        int left=currentCellLeft(cellIndex, leafIndex, logicalLeafIndex, sectionRect.left(), hv);
                        int width=currentCellWidth(cellIndex, leafIndex, logicalLeafIndex, hv);

                        QRect r(left, top, width, height);

                        uniopt.text = cellIndex.data(Qt::DisplayRole).toString();

                        if(cellIndex.data(Qt::UserRole).isValid())
                        {
                                QRect new_r(0, 0,  r.width(), r.height());
                                uniopt.rect = new_r;
                                QPixmap original_button(r.width(), r.height());
                                QPainter original_button_painter(&original_button);
                                hv->style()->drawControl(QStyle::CE_HeaderSection, &uniopt, &original_button_painter, hv);

                                new_r.setWidth(r.height());
                                new_r.setHeight(r.width());
                                QMatrix m;
                                m.translate(0, r.height());
                                m.rotate(-90);
                                original_button_painter.setWorldMatrix(m, true);
                                uniopt.rect = new_r;
                                hv->style()->drawControl(QStyle::CE_HeaderLabel, &uniopt, &original_button_painter, hv);

                                painter->drawPixmap(r, original_button);
                        }
                        else
                        {
                                bool clipping=painter->hasClipping();
                                uniopt.rect = r;
                                hv->style()->drawControl(QStyle::CE_Header, &uniopt, painter, hv);
                                painter->setClipping(clipping);
                        }
                        return top+height;
                }



                void paintHorizontalSection
                        (QPainter *painter,
                         const QRect& sectionRect,
                         int logicalLeafIndex,
                         const QHeaderView* hv,
                         const QStyleOptionHeader& styleOptions,
                         const QModelIndex& leafIndex) const
                {
                        QPointF oldBO(painter->brushOrigin());
                        int top=sectionRect.y();
                        QModelIndexList indexes(parentIndexes(leafIndex));
                        for(int i=0; i<indexes.size(); ++i)
                        {
                                top=paintHorizontalCell(painter,
                                               hv,
                                                           indexes[i],
                                                           leafIndex,
                                                           logicalLeafIndex,
                                                           styleOptions,
                                                           sectionRect,
                                                           top);
                        }
                        painter->setBrushOrigin(oldBO);
                }

};


HierarchicalHeaderView::HierarchicalHeaderView(Qt::Orientation orientation, QWidget *parent)
        :QHeaderView(orientation, parent), _pd(new private_data())
{
        connect(this, SIGNAL(sectionResized(int, int, int)), this, SLOT(on_sectionResized(int)));
}



HierarchicalHeaderView::~HierarchicalHeaderView()
{
        delete _pd;
}


QStyleOptionHeader HierarchicalHeaderView::styleOptionForCell(int logicalInd) const
{
        QStyleOptionHeader opt;
        initStyleOption(&opt);
        if (window()->isActiveWindow())
                opt.state |= QStyle::State_Active;
        opt.textAlignment = Qt::AlignCenter;
        opt.iconAlignment = Qt::AlignVCenter;
        opt.section = logicalInd;

        int visual = visualIndex(logicalInd);

        if (count() == 1)
                opt.position = QStyleOptionHeader::OnlyOneSection;
        else
        {
                if (visual == 0)
                        opt.position = QStyleOptionHeader::Beginning;
                else
                        opt.position=(visual==count()-1 ? QStyleOptionHeader::End : QStyleOptionHeader::Middle);
        }

    if(isClickable())
        {
                {
                        if(highlightSections() && selectionModel())
                        {
                                if(selectionModel()->columnIntersectsSelection(logicalInd, rootIndex()))
                                        opt.state |= QStyle::State_On;
                                if(selectionModel()->isColumnSelected(logicalInd, rootIndex()))
                                        opt.state |= QStyle::State_Sunken;
                        }
        }
    }
        if(selectionModel())
        {
            bool previousSelected = selectionModel()->isColumnSelected(logicalIndex(visual - 1), rootIndex());
                bool nextSelected =  selectionModel()->isColumnSelected(logicalIndex(visual + 1), rootIndex());
                if (previousSelected && nextSelected)
                        opt.selectedPosition = QStyleOptionHeader::NextAndPreviousAreSelected;
                else
                {
                        if (previousSelected)
                                opt.selectedPosition = QStyleOptionHeader::PreviousIsSelected;
                        else
                        {
                                if (nextSelected)
                                        opt.selectedPosition = QStyleOptionHeader::NextIsSelected;
                                else
                                        opt.selectedPosition = QStyleOptionHeader::NotAdjacent;
                        }
                }
        }
        return opt;
}

QSize HierarchicalHeaderView::sectionSizeFromContents(int logicalIndex) const
{
        if(_pd->headerModel)
        {
                QModelIndex curLeafIndex(_pd->leafIndex(logicalIndex));
                if(curLeafIndex.isValid())
                {
                        QStyleOptionHeader styleOption(styleOptionForCell(logicalIndex));
                        QSize s(_pd->cellSize(curLeafIndex, this, styleOption));
                        curLeafIndex=curLeafIndex.parent();
                        while(curLeafIndex.isValid())
                        {
                                if(orientation() == Qt::Horizontal)
                                        s.rheight()+=_pd->cellSize(curLeafIndex, this, styleOption).height();
                                curLeafIndex=curLeafIndex.parent();
                        }
                        return s;
                }
        }
        return QHeaderView::sectionSizeFromContents(logicalIndex);
}


void HierarchicalHeaderView::paintSection
        (QPainter *painter,
         const QRect &rect,
         int logicalIndex) const
{
        if (rect.isValid())
        {
                QModelIndex leafIndex(_pd->leafIndex(logicalIndex));
                if(leafIndex.isValid())
                {
                        if(orientation() == Qt::Horizontal)
                                _pd->paintHorizontalSection(painter, rect, logicalIndex, this, styleOptionForCell(logicalIndex), leafIndex);
                        return;
                }
        }
        QHeaderView::paintSection(painter, rect, logicalIndex);
}

void HierarchicalHeaderView::on_sectionResized(int logicalIndex)
{
        if(isSectionHidden(logicalIndex))
                return;

        QModelIndex leafIndex(_pd->leafIndex(logicalIndex));
        if(leafIndex.isValid())
        {
                QModelIndexList leafsList(_pd->leafs(_pd->findRootIndex(leafIndex)));
                for(int n=leafsList.indexOf(leafIndex); n>0; --n)
                {
                        --logicalIndex;

                        int w = viewport()->width();
                        int h = viewport()->height();
                        int pos = sectionViewportPosition(logicalIndex);
                        QRect r(pos, 0, w - pos, h);
                        if(orientation() == Qt::Horizontal)
                        {
                                if (isRightToLeft())
                                        r.setRect(0, 0, pos + sectionSize(logicalIndex), h);
                        }
                        else
                                r.setRect(0, pos, w, h - pos);

                        viewport()->update(r.normalized());
                }
        }
}

void HierarchicalHeaderView::setModel(QAbstractItemModel* model)
{
        _pd->initFromNewModel(model);
        QHeaderView::setModel(model);
        int cnt=model->columnCount();
        if(cnt)
                initializeSections(0, cnt-1);
}


HideShowColumnsCTV::HideShowColumnsCTV(QList <ColumnInfoCTV *> *columnList, QWidget *parent) :
    QDialog(parent)
{
    this->columnList = columnList;
    window()->setWindowFlags(Qt::WindowCloseButtonHint);
    setWindowTitle(tr("Скрыть/Отобразить колонки таблицы"));
    this->setFixedSize(800, 600);

    hBottomLayout = 0;
    gridLayout = 0;
    vMainLayout = 0;
    setLayout(getVMainLayout());
    window()->layout()->setSizeConstraint(QLayout::SetFixedSize);
}

QVBoxLayout * HideShowColumnsCTV::getVMainLayout()
{
    vMainLayout = new QVBoxLayout;
    vMainLayout->addLayout(getGridLayoutWithTable());
    vMainLayout->addLayout(getHBottomLayout());
    vMainLayout->addStretch();
    return vMainLayout;
}

QHBoxLayout * HideShowColumnsCTV::getHBottomLayout()
{
    OKPushButton = new QPushButton(tr("OK"));
    connect(OKPushButton, SIGNAL(pressed()), this, SLOT(fixedColumns()));
    cancelPushButton = new QPushButton(tr("Отмена"));
    connect(cancelPushButton, SIGNAL(pressed()), this, SLOT(reject()));
    hBottomLayout = new QHBoxLayout;
    hBottomLayout->addStretch();
    hBottomLayout->addWidget(OKPushButton);
    hBottomLayout->addWidget(cancelPushButton);
    return hBottomLayout;
}

void HideShowColumnsCTV::fixedColumns()
{
    QCheckBox * checkBox;
    for (int i=0; i<columnList->size(); ++i)
    {
        checkBox = static_cast<QCheckBox *>(tableWidget->cellWidget(i,0));
        columnList->at(i)->hidden = checkBox->isChecked();
    }
    accept();
}

QGridLayout * HideShowColumnsCTV::getGridLayoutWithTable()
{
    tableWidget = new QTableWidget;
    tableWidget->setVisible(true);
    tableWidget->setColumnCount(1);
    tableWidget->setRowCount(columnList->size());
    tableWidget->setAlternatingRowColors(true);
    tableWidget->setColumnWidth(0, 750);
    QTableWidgetItem * item = new QTableWidgetItem(tr("Колонки таблицы"));
    tableWidget->setHorizontalHeaderItem(0, item);
    QCheckBox * checkBox;
    for (int i=0; i<columnList->size(); ++i)
    {
        checkBox = new QCheckBox;
        checkBox->setChecked(columnList->at(i)->hidden);
        checkBox->setText(columnList->at(i)->columnName);
        tableWidget->setCellWidget(i,0,checkBox);
        if ( columnList->at(i)->frozen )
            checkBox->setEnabled(false);
    }
    gridLayout = new QGridLayout;
    gridLayout->addWidget(tableWidget, 0, 0);
    return gridLayout;
}

QgebComplexTableView::QgebComplexTableView(QWidget *parent)
    : QTableView(parent)
{
    init(parent);
}

void QgebComplexTableView::init(QWidget *parent)
{
    this->parent = parent;
    frozenCount = -1;
    title = "";
    headerModel = 0;
    frTblView = 0;
    proxyModel = new ProxyModelWithHeaderModels;
    hideColumnAction = new QAction(tr("Скрыть/Отобразить колонки"), this);
    hideColumnAction->setStatusTip(tr("Скрыть/Отобразить колонки таблицы"));
    hideColumnAction->setEnabled(true);
    connect(hideColumnAction, SIGNAL(triggered()), this, SLOT(hideColumn()));
    outHtmlAction = new QAction(tr("Сохранить в HTML"), this);
    outHtmlAction->setStatusTip(tr("Сохранить таблицу в HTML"));
    outHtmlAction->setEnabled(true);
    connect(outHtmlAction, SIGNAL(triggered()), this, SLOT(outHtml()));
    printTableAction = new QAction(tr("Печать"), this);
    printTableAction->setStatusTip(tr("Печать таблицы"));
    printTableAction->setEnabled(true);
    connect(printTableAction, SIGNAL(triggered()), this, SLOT(printTable()));
    columnList = new QList<ColumnInfoCTV *>;
 }


QgebComplexTableView::~QgebComplexTableView()
{
    if (headerModel != 0)
        delete headerModel;
}


void QgebComplexTableView::setHeaderModel(QStandardItemModel *headerModel)
{
    this->headerModel = headerModel;
    buildColumnList();
}


void QgebComplexTableView::setModel(QAbstractItemModel *model)
{
    proxyModel->setModel(model);
    proxyModel->setHorizontalHeaderModel(headerModel);
    this->aiModel = proxyModel;
    QTableView::setModel(proxyModel);
    hv = new HierarchicalHeaderView(Qt::Horizontal, this);
    setHorizontalHeader(hv);
}


void QgebComplexTableView::setHorizontalHeader(HierarchicalHeaderView* hierarchicalHeaderView)
{
    QTableView::setHorizontalHeader(hierarchicalHeaderView);
    this->hierarchicalHeaderView = hierarchicalHeaderView;
    hierarchicalHeaderView->addAction(hideColumnAction);
#ifndef QT_NO_PRINTDIALOG
    hierarchicalHeaderView->addAction(printTableAction);
#endif
    hierarchicalHeaderView->addAction(outHtmlAction);
    hierarchicalHeaderView->setContextMenuPolicy(Qt::ActionsContextMenu);
}

void QgebComplexTableView::setFrozenColumn(int frozenCount)
{
    this->frozenCount = frozenCount;
    frTblView = new QTableView(this);
    frTblView->setModel(model());
    frTblView->setFocusPolicy(Qt::NoFocus);
    frTblView->verticalHeader()->hide();

    hv = new HierarchicalHeaderView(Qt::Horizontal, frTblView);
    frTblView->setHorizontalHeader(hv);
    this->viewport()->stackUnder(frTblView);
    frTblView->setStyleSheet("QTableView { border: none; }" );
    frTblView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    frTblView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    updateFrozenTableGeometry();
    setHorizontalScrollMode(ScrollPerPixel);
    setVerticalScrollMode(ScrollPerPixel);
    frTblView->setVerticalScrollMode(ScrollPerPixel);
    for (int col=0; col<frozenCount; ++col)
        frTblView->setColumnWidth(col, columnWidth(col) );
    frTblView->setSelectionModel(selectionModel());
    frTblView->setAutoScroll(false);
    connect(horizontalHeader(),SIGNAL(sectionResized ( int ,int,int )),
             this, SLOT(updateSectionWidth(int, int, int)));
    connect(frTblView->horizontalHeader(),SIGNAL(sectionResized ( int ,int,int )),
             this, SLOT(updateSectionWidthFromFix(int, int, int)));
    connect(verticalHeader(),SIGNAL(sectionResized ( int ,int,int )),
             this, SLOT(updateSectionHeight(int, int, int)));
    connect(frTblView->verticalScrollBar(),
             SIGNAL(valueChanged(int)),
            verticalScrollBar(), SLOT(setValue(int)));
    connect(verticalScrollBar(), SIGNAL(valueChanged(int)),
            frTblView->verticalScrollBar(), SLOT(setValue(int)));
    frTblView->horizontalHeader()->addAction(hideColumnAction);
    frTblView->horizontalHeader()->addAction(outHtmlAction);
#ifndef QT_NO_PRINTDIALOG
    frTblView->horizontalHeader()->addAction(printTableAction);
#endif
    frTblView->horizontalHeader()->setContextMenuPolicy(Qt::ActionsContextMenu);
    buildColumnList();

 }

void QgebComplexTableView::setSelectedFixedRow(QModelIndex index)
{
   if (frozenCount > 0)
        frTblView->selectRow(index.row());
    selectRow(index.row());
}

void QgebComplexTableView::updateSectionWidth(int logicalIndex, int, int newSize)
 {
    if(logicalIndex < frozenCount)
    {
        if (frozenCount > 0)
            frTblView->setColumnWidth(logicalIndex,newSize);
        updateFrozenTableGeometry();
    }
 }


void QgebComplexTableView::updateSectionWidthFromFix(int logicalIndex, int, int newSize)
 {
     setColumnWidth(logicalIndex,newSize);
     updateFrozenTableGeometry();
 }

void QgebComplexTableView::updateSectionHeight(int logicalIndex, int,
                                               int newSize)
 {
     if (frozenCount > 0)
        frTblView->setRowHeight(logicalIndex, newSize);
 }

 void QgebComplexTableView::updateFrozenTableGeometry()
  {
     int w = 0;
     for (int col=0; col<frozenCount; ++col)
     {
         w = w + columnWidth(col);
     }
     if ( frTblView > 0 )
         frTblView->setGeometry( verticalHeader()->width()+frameWidth(),
             frameWidth(), w, viewport()->height()+horizontalHeader()->height());
  }


 void QgebComplexTableView::resizeEvent(QResizeEvent * event)
  {
        QTableView::resizeEvent(event);
        if (frozenCount > 0)
            updateFrozenTableGeometry();
   }


 void QgebComplexTableView::printTable()
 {
#ifndef QT_NO_PRINTDIALOG
     QString s;
     if ( textHtml.isEmpty() )
     {
         out.setString(&s, QIODevice::ReadWrite);
         createHtmlDocument();
     }
     else
         s = textHtml;
     QWebView *webView = new QWebView(this);
     webView->setVisible(false);
     webView->setHtml(s.toAscii());

    QPrinter printer;
    QPrintPreviewDialog *prevDlg = new QPrintPreviewDialog(&printer, this);
    QIcon icon(this->windowIcon());
    prevDlg->setWindowIcon(icon);
    connect(prevDlg, SIGNAL(paintRequested(QPrinter*)), webView, SLOT(print(QPrinter*)));
    prevDlg->exec();
    delete webView;
 #endif

 }

 void QgebComplexTableView::outHtml()
 {
      QFileDialog::Options options;
      QString selectedFilter;
      QString fileName = QFileDialog::getSaveFileName(this,
                                  tr("Файл сохранения"),
                                  "",
                                  tr("HTML файлы (*.html)"),
                                  &selectedFilter,
                                  options);
      if (fileName.isEmpty())
          return;
     QFile file(fileName);
     if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
              return;
     out.setDevice(&file);
     createHtmlDocument();
     file.close();
 }

 void QgebComplexTableView::createHtmlDocument()
 {
     int colSpan;
     int rowSpan;
     QString s;
     QStandardItem *standardItem;
     QStandardItem *standardItem1;
     QList <QStandardItem *> *listStandardItem = new QList <QStandardItem *>;
     QList <QStandardItem *> *listStandardItem1 = new QList <QStandardItem *>;
     out << "<head>\n";
     out << "<meta http-equiv='Content-Type' content='text/html; charset=UTF-8' />\n";
     out << tr("<title>HTML представление таблицы</title>\n");
     out << "</head>\n";
     out << "<body>\n";
     if ( !title.isEmpty() )
         out << "<h2 align='center'>" + tr(title.toAscii()) + "</h2>\n";
     out << "<table width='" << this->width() << "' border='1' cellspacing='0' cellpadding='0'>\n";
     if (headerModel == 0)
     {
         out << "<tr>\n";
         for (int col=0; col<model()->columnCount(); ++col)
         {
             out << "<th align='center' valign='center' " + setHtmlFont(this->horizontalHeader()) + ">";
             s = ( horizontalHeader()->model()->headerData(col, Qt::Horizontal).toString().isNull() ) ? "&nbsp" :
                     tr(horizontalHeader()->model()->headerData(col, Qt::Horizontal).toString().replace("\n", "<br/>").toAscii());
             out << s + "</th>\n";
         }
         out << "</tr>\n";
     }
     else
     {
         bathos = getBathosModel();
         for (int row=0; row<bathos; row++)
         {
             out << "<tr>\n";
             if (row == 0)
             {
                 for(int col=0; col<headerModel->columnCount(QModelIndex());++col)
                 {
                     standardItem = headerModel->item(row, col);
                     if (standardItem->columnCount() > 0)
                     {
                         listStandardItem1->append(standardItem);
                         colSpan = getColumnSpan(standardItem);
                         rowSpan = bathos - row - buildBathos(standardItem, 0, 0);
                     }
                     else
                     {
                        colSpan = 1;
                        rowSpan = bathos - row - buildBathos(standardItem, 0, 0);
                    }
                     out << "<th rowSpan='" + QString::number(rowSpan) + "' colSpan='" + QString::number(colSpan)
                             + "'" + setHtmlAlign(standardItem) + " " + setHtmlFont(standardItem)+ ">";
                     s = ( standardItem->text().isNull() ) ? "&nbsp" :
                             tr(standardItem->text().toAscii().replace("\n", "<br/>"));
                     out << s + "</th>\n";
                 }
             }
             else
             {
                 listStandardItem->clear();
                 for (int i=0; i<listStandardItem1->size(); ++i)
                 {
                     listStandardItem->append(listStandardItem1->at(i));
                 }
                 listStandardItem1->clear();
                 for(int list=0; list<listStandardItem->size();++list)
                 {
                     standardItem1 = listStandardItem->at(list);
                     for (int col=0; col<standardItem1->columnCount(); ++col)
                     {
                         standardItem = standardItem1->child(0, col);
                         if (standardItem->columnCount() > 0)
                         {
                             listStandardItem1->append(standardItem);
                             colSpan = getColumnSpan(standardItem);
                             rowSpan = bathos - row - buildBathos(standardItem, 0, 0);
                         }
                         else
                         {
                             colSpan = 1;
                             rowSpan = bathos - row - buildBathos(standardItem, 0, 0);
                         }
                         out << "<th rowSpan='" + QString::number(rowSpan) + "' colSpan='" + QString::number(colSpan)
                                 + "'" + setHtmlAlign(standardItem) + " " + setHtmlFont(standardItem)+ ">";
                         s = ( standardItem->text().isNull() ) ? "&nbsp" :
                                 tr(standardItem->text().toAscii().replace("\n", "<br/>"));
                         out << s + "</th>\n";
                     }
                 }
             }
             out << "</tr>\n";
         }
     }
     for (int row=0; row<this->aiModel->rowCount(); ++row)
     {
         out << "<tr>\n";
         for (int col=0; col<this->aiModel->columnCount(); ++col)
         {
             out << "<td" + setHtmlAlign(col) + setHtmlFont() + " width='" + columnWidth(col) + "'>";
             s = ( model()->index(row, col, QModelIndex()).data().toString().isEmpty() ) ? "&nbsp" :
                     model()->index(row, col, QModelIndex()).data().toString().replace("\n", "<br/>") ;
             out << s + "</td>\n";
         }
         out << "</tr>\n";
     }
     out << "</table>\n";
     out << "</body>\n";
     out << "</html>\n";
 }

 QString QgebComplexTableView::setHtmlAlign(int col)
 {
     QString ha = "align='";
     QString va = "valign='";
     int al;
     if ( (alignColumnText.isEmpty()) || (col > alignColumnText.count()) )
         ha = ha + "left'";
     else
     {
         al = alignColumnText.at(col);
         switch ( al )
         {
         case Qt::AlignLeft:
             ha = ha + "left'";
             break;
         case Qt::AlignRight:
             ha = ha + "right'";
             break;
         case Qt::AlignCenter:
             ha = ha + "center'";
             break;
         case Qt::AlignHCenter:
             ha = ha + "center'";
             break;
         case Qt::AlignJustify:
             ha = ha + "justify'";
             break;
         default:
             ha = ha + "left'";
             break;
         }
     }
     if ( (valignColumnText.isEmpty()) || (col > valignColumnText.count()) )
         va = va + "top'";
     else
     {
         al = valignColumnText.at(col);
         switch ( al )
         {
         case Qt::AlignTop:
             va = va + "top'";
             break;
         case Qt::AlignBottom:
             va = va + "bottom'";
             break;
         case Qt::AlignCenter:
             va = va + "middle'";
             break;
         case Qt::AlignVCenter:
             va = va + "middle'";
             break;
         default:
             va = va + "top'";
             break;
         }
     }
     return " " + ha + " " + va;
 }

 QString QgebComplexTableView::setHtmlAlign(QStandardItem * standardItem)
 {
     int al = standardItem->textAlignment() & 0x000f;
     QString ha = "align='";
     switch ( al )
     {
     case Qt::AlignLeft:
         ha = ha + "left'";
         break;
     case Qt::AlignRight:
         ha = ha + "right'";
         break;
     case Qt::AlignCenter:
         ha = ha + "center'";
         break;
     case Qt::AlignJustify:
         ha = ha + "justify'";
         break;
     default:
         ha = ha + "center'";
         break;
     }
     al = standardItem->textAlignment() & 0x00f0;
     QString va = "valign='";
     switch ( al )
     {
     case Qt::AlignTop:
         va = va + "top'";
         break;
     case Qt::AlignBottom:
         va = va + "bottom'";
         break;
     case Qt::AlignVCenter:
         va = va + "middle'";
         break;
     default:
         va = va + "middle'";
         break;
     }
     return " " + ha + " " + va;
 }

 QString QgebComplexTableView::setHtmlFont()
 {
     QString font ="style='{font:";
     if ( this->font().bold() )
         font = font + " " + "bold";
     if ( this->font().italic() )
         font = font + " " + "italic";
     if ( this->font().underline() )
         font = font + " " + "normal";
     font = font + " " + QString::number(this->font().pointSize()) + "pt" +
                   " " + this->font().family() + "}'";
     return font;
 }

 QString QgebComplexTableView::setHtmlFont(QHeaderView * horizontalHeader)
 {
     QString font ="style='{font:";
     if ( horizontalHeader->font().bold() )
         font = font + " " + "bold";
     if ( horizontalHeader->font().italic() )
         font = font + " " + "italic";
     if ( horizontalHeader->font().underline() )
         font = font + " " + "normal";
     font = font + " " + QString::number(horizontalHeader->font().pointSize()) + "pt" +
                   " " + horizontalHeader->font().family() + "}'";
     return font;
 }

QString QgebComplexTableView::setHtmlFont(QStandardItem * standardItem)
{
    QString font ="style='{font:";
    if ( standardItem->font().bold() )
        font = font + " " + "bold";
    if ( standardItem->font().italic() )
        font = font + " " + "italic";
    if ( standardItem->font().underline() )
        font = font + " " + "underline";
    font = font + " " + QString::number(standardItem->font().pointSize()) + "pt" +
                  " " + standardItem->font().family() + "}'";
    return font;
}

 int QgebComplexTableView::getColumnSpan(QStandardItem *item)
 {
     QStandardItem *standardItem;
     int columnSpan = 0;
     int col;
     for (int i=0; i<item->columnCount(); ++i)
     {
         standardItem = item->child(0, i);
         col = item->child(0, i)->columnCount();
         if (col > 0)
             col = getColumnSpan(item->child(0, i));
         else
             col = 1;
         columnSpan = columnSpan + col;
     }
     return columnSpan;
 }

 int QgebComplexTableView::getBathosModel()
 {
     QStandardItem *standardItem;
     int maxBathos = 0;
     int curBathos = 0;
     for (int i=0; i<headerModel->columnCount(QModelIndex()); ++i)
     {
        standardItem = headerModel->item(0, i);
        if (standardItem->columnCount() != 0)
        {
            curBathos = buildBathos(standardItem, 0, 0);
            if (curBathos > maxBathos)
                maxBathos = curBathos;
        }
     }
     return maxBathos;
 }


 int QgebComplexTableView::buildBathos(QStandardItem *standardItem, int beforeBathos, int maxBathos)
 {
     QStandardItem *si;
     int curBathos;
     int maxxBathos = maxBathos;
     for (int i=0; i<standardItem->columnCount(); ++i)
     {
         si = standardItem->child(0, i);
         if (si->columnCount() == 0)
         {
             curBathos = beforeBathos + 1;
         }
         else
         {
             curBathos = beforeBathos = buildBathos(si, beforeBathos + 1, maxBathos + 1);
         }
         if (curBathos > maxxBathos)
             maxxBathos = curBathos;
     }
     return maxxBathos;
 }

 void QgebComplexTableView::hideColumn()
 {
     QStandardItem *standardItem;
     if (headerModel == 0)
     {
         for (int i=0; i<aiModel->columnCount(); ++i)
         {
             columnList->at(i)->frozen = ( i < frozenCount ) ? true : false;
             columnList->at(i)->hidden = isColumnHidden(i);
             columnList->at(i)->columnWidth = this->columnWidth(i);
         }
     }
     else
     {
         hierColumn = -1;
         for (int i=0; i<headerModel->columnCount(QModelIndex()); ++i)
         {
            standardItem = headerModel->item(0, i);
            hierColumn = hierColumn + 1;
            if ( i < frozenCount )
            {
                columnList->at(hierColumn)->frozen = true;
                columnList->at(hierColumn)->hidden = false;
                columnList->at(hierColumn)->columnWidth = columnWidth(i);
                columnList->at(hierColumn)->columnName = standardItem->text();
            }
            else
            {
                if (standardItem->columnCount() == 0)
                {
                    columnList->at(hierColumn)->frozen = false;
                    columnList->at(hierColumn)->hidden = isColumnHidden(i);
                    columnList->at(hierColumn)->columnWidth = columnWidth(i);
                    columnList->at(hierColumn)->columnName = standardItem->text();
                }
                else
                {
                    hierColumn = hierColumn - 1;
                    buildList1(standardItem);
                }
            }
        }
     }
     HideShowColumnsCTV *hiddenColumnDialog = new HideShowColumnsCTV(columnList);
     QIcon icon(this->windowIcon());
     hiddenColumnDialog->setWindowIcon(icon);
     hiddenColumnDialog->exec();
     for (int i=0; i<columnList->size()-1; ++i)
     {
         columnList->at(i)->columnWidth = 0;
         if ( columnList->at(i)->hidden )
             columnList->at(i)->columnWidth = this->columnWidth(i);
         setColumnHidden(i, columnList->at(i)->hidden);
     }
 }

void QgebComplexTableView::buildList1(QStandardItem *standardItem)
{
    QStandardItem *si;
    for (int i=0; i<standardItem->columnCount(); ++i)
    {
        si = standardItem->child(0, i);
        hierColumn = hierColumn + 1;
        if (si->columnCount() == 0)
        {
            columnList->at(hierColumn)->hidden = isColumnHidden(hierColumn);
            columnList->at(hierColumn)->columnWidth = columnWidth(hierColumn);
            columnList->at(hierColumn)->columnName = si->text();
            columnList->at(hierColumn)->frozen = false;
         }
        else
        {
            hierColumn = hierColumn - 1;
            buildList1(si);
        }
    }
}

void QgebComplexTableView::buildList(QStandardItem *standardItem)
{
   QStandardItem *si;
   ColumnInfoCTV *columnInfo;
   for (int i=0; i<standardItem->columnCount(); ++i)
   {
       si = standardItem->child(0, i);
       if (si->columnCount() == 0)
       {
           hierColumn = hierColumn + 1;
           columnInfo = new ColumnInfoCTV;
           columnInfo->columnName = si->text();
           columnInfo->columnNumber = hierColumn;
           columnInfo->frozen = false;
           columnInfo->hidden = false;
           columnInfo->columnWidth = 0;
           columnList->append(columnInfo);
       }
       else
       {
           buildList(si);
       }
   }
}


void QgebComplexTableView::buildColumnList()
{
    QStandardItem *standardItem;
    ColumnInfoCTV *columnInfo;
    columnList->clear();
    if ( (headerModel != 0) )
    {
        hierColumn = - 1;
        for (int i=0; i<headerModel->columnCount(QModelIndex()); ++i)
        {
           standardItem = headerModel->item(0, i);
           if (standardItem->columnCount() == 0)
           {
               hierColumn = hierColumn + 1;
               columnInfo = new ColumnInfoCTV;
               columnInfo->columnName = standardItem->text();
               columnInfo->columnNumber = hierColumn;
               if (i < frozenCount)
                   columnInfo->frozen = true;
               else
                   columnInfo->frozen = false;
               columnInfo->hidden = false;
               columnInfo->columnWidth = 0;
               columnList->append(columnInfo);
           }
           else
           {
               buildList(standardItem);
           }
        }
    }
    else
    {
        for (int i=0; i<frTblView->model()->columnCount(); ++i)
        {
            columnInfo = new ColumnInfoCTV;
            columnInfo->columnName = horizontalHeader()->model()->headerData(i, Qt::Horizontal).toString();
            columnInfo->columnNumber = i;
            if (i < frozenCount)
                columnInfo->frozen = true;
            else
                columnInfo->frozen = false;
            columnInfo->hidden = frTblView->isColumnHidden(i);
            columnInfo->columnWidth = 0;
            columnList->append(columnInfo);
        }
    }
}


int QgebComplexTableView::saveStateTable(QString path)
{
    QString pth = path;
    if ( objectName().isEmpty() )
        return -1;
    if ( parent->objectName().isEmpty() )
        return -2;
    doc = new QDomDocument("document");
    pth.replace(QString("\\"), QString("/"));
    if ( QFile::exists(pth) )
    {
        QFile * file = new QFile(pth);

        if (file->open(QIODevice::ReadOnly))
        {
            if (!doc->setContent(file, true, &eStr, &eLine, &eCol))
            {
                eMes = tr("line=") + QString::number(eLine) + tr(", position=") + QString::number(eCol);
                file->close();
                return -4;
            }
            file->close();
        }
        else
            return -5;
    }
    return saveTable(pth);
}

int QgebComplexTableView::saveTable(QString pth)
{
    bool flagFirst = true;

    QDomElement docElem;
    if (doc->hasChildNodes())
    {
        docElem = doc->documentElement();
    }
    else
    {
        docElem = doc->createElement("states");
        doc->appendChild(docElem);
    }
    QDomNode n = docElem.firstChild();
    QDomNode nCol, nColNext;
    QString s;
    QDomElement dec;
    QDomElement de;
    try
    {
        while(!n.isNull())
        {
            QDomElement e = n.toElement();
            if(!e.isNull())
            {
                if (e.tagName() == "parametrs")
                {
                    if ((e.attributeNode("name").value() == parent->objectName() )
                        and (e.attributeNode("table").value() == objectName()))
                    {
                        nCol = n.firstChild();
                        while(!nCol.isNull())
                        {
                            nColNext = nCol.nextSibling();
                            n.removeChild(nCol);
                            nCol = nColNext;
                        }
                        de = e;
                        flagFirst = false;
                    }
                }
            }
            n = n.nextSibling();
        }
        if (flagFirst == true)
        {
            de = doc->createElement("parametrs");
            de.setAttribute("name", parent->objectName());
            de.setAttribute("table", objectName());
            docElem.appendChild(de);
        }
        for(int i=0; i<this->model()->columnCount(); ++i)
        {
            dec = doc->createElement("col");
            dec.setAttribute("num", QString::number(i));
            if ( isColumnHidden(i) == true )
            {
                dec.setAttribute("show", "false");
                dec.setAttribute("width", QString::number(columnList->at(i)->columnWidth));
            }
            else
            {
                dec.setAttribute("show", "true");
                dec.setAttribute("width", QString::number(columnWidth(i)));
            }
            de.appendChild(dec);
        }
        return save(pth);
    }
    catch(...)
    {
        return -6;
    }
}

int QgebComplexTableView::save(QString pth)
{
    try
    {
        QFile * file = new QFile(pth);
        if (file->open(QIODevice::WriteOnly | QIODevice::Truncate))
        {
            QTextStream out(file);
            QDomElement docElem = doc->documentElement();
            docElem.save(out, 3);
            file->close();
            return 0;
        }
        else
        {
            file->close();
            return -7;
        }
    }
    catch(...)
    {
        return -6;
    }
}

int QgebComplexTableView::restoreStateTable(QString path)
{
    QString pth = path;
    if ( objectName().isEmpty() )
        return -1;
    if ( parent->objectName().isEmpty() )
        return -2;
    doc = new QDomDocument("document");
    pth.replace(QString("\\"), QString("/"));
    if ( !QFile::exists(pth) )
        return -3;
    QFile * file = new QFile(pth);

    if (file->open(QIODevice::ReadWrite))
    {
        if (!doc->setContent(file, true, &eStr, &eLine, &eCol))
        {
            eMes = tr("line=") + QString::number(eLine) + tr(", position=") + QString::number(eCol);
            file->close();
            return -4;
        }
        file->close();
    }
    else
        return -5;
    QDomElement docElem = doc->documentElement();
    QDomNode n = docElem.firstChild();
    QDomNode nCol;
    int c, w;
    QString s;
    try
    {
        while(!n.isNull())
        {
            QDomElement e = n.toElement();
            if(!e.isNull())
            {
                if (e.tagName() == "parametrs")
                {
                    if ((e.attributeNode("name").value() == parent->objectName())
                        and (e.attributeNode("table").value() == objectName()))
                    {
                        nCol = n.firstChild();
                        columnList->clear();
                        ColumnInfoCTV * columnInfo;
                        while(!nCol.isNull())
                        {
                            c = nCol.toElement().attributeNode("num").value().toInt();
                            w = nCol.toElement().attributeNode("width").value().toInt();
                            s = nCol.toElement().attributeNode("show").value();
                            setColumnWidth(c, w);
                            if (s == "true")
                                setColumnHidden(c, false);
                            else
                                setColumnHidden(c, true);
                            columnInfo = new ColumnInfoCTV;
                            columnInfo->columnName = model()->headerData(c, Qt::Horizontal, Qt::DisplayRole).toString();
                            columnInfo->columnNumber = c;
                            columnInfo->hidden = isColumnHidden(c);
                            columnInfo->columnWidth = w;
                            columnList->append(columnInfo);
                            nCol = nCol.nextSibling();
                        }
                    }
                }
            }
            n = n.nextSibling();
        }
        return true;
    }
    catch(...)
    {
        return -6;
    }
}

void QgebComplexTableView::setAlignColumn(QVector <int> align)
{
    alignColumnText = align;
}

void QgebComplexTableView::setVAlignColumn(QVector <int> valign)
{
    valignColumnText = valign;
}
